package services;

import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import l2p.gameserver.ThreadPoolManager;
import l2p.gameserver.database.DatabaseFactory;
import l2p.gameserver.instancemanager.ServerVariables;
import l2p.gameserver.model.L2ObjectsStorage;
import l2p.gameserver.model.L2Player;
import l2p.gameserver.model.quest.Quest;
import l2p.gameserver.scripts.ScriptFile;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStreamReader;
import java.lang.reflect.Type;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Paths;
import java.sql.*;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.util.Date;
import java.util.List;
import java.util.Random;
import java.util.logging.*;
import java.util.logging.Level;

public class l2vote implements ScriptFile {
    public static  String API_URL_TEMPLATE = "https://api.l2vote.com/api/votes/server/%s/votes/today";
    public static  String API_KEY="";                                           // Укажите ваш API-ключ
    public static  int SERVER_ID=0;                                             // ID сервера
    public static  int POLLING_DELAY_MINUTES=1;                                 // Вермя задержки опроса в минутах
    public static String[] L2VoteRewardItemId = "57:1".split(";");              // Предметы за голос, можно указывать несколько, пример - 57:1;4037:1
    public static boolean L2VoteRewardItemRandom=false;                         // Если включить, выдаст рандомно только 1 награду из конфига L2VoteRewardItemId
    public static boolean ALLOW_MULTIPLE_REWARDS=false;                         // Можно голосовать на 1 ник множнство раз   
    public static boolean ENABLE_CHECK_IP=false;                                // Включить проверку по IP
    public static boolean ENABLE_CHECK_HWID=false;                              // Включить проверку по HWID

    @Override
    public void onLoad() {
        voteProcessor = new VoteProcessor();
        voteProcessor.onLoad();
    }

    @Override
    public void onReload() {

    }

    @Override
    public void onShutdown() {

    }
    public static VoteProcessor voteProcessor;



    public class VoteProcessor {

        private String today;

        public VoteProcessor() {
            SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd");

            this.today = sdf.format(new Date());

        }

        public void start() {
            Connection connection = null;
            DatabaseManager dbManager = null;
            try {
                connection = DatabaseFactory.getInstance().getConnection();
                dbManager = new DatabaseManager(connection);

                // Создаем таблицу
                dbManager.createTableIfNotExists();

                // Получаем данные из API
                ApiClient apiClient = new ApiClient();
                List<Vote> votes = apiClient.fetchVotes();

                // Обрабатываем каждый голос
                for (Vote vote : votes) {
                    if (dbManager.addVote(vote.getCharName(), vote.getVoteTime(), vote.getId())) {
                        LogManagerVote.getInstance().info("Добавлен новый голос: " + vote);
                    }
                }

                // Выдаем награды
                dbManager.giveRewards(ALLOW_MULTIPLE_REWARDS);

            } catch (Exception e) {
                e.printStackTrace();
            } finally {
                if (dbManager != null) {
                    try {
                        dbManager.close();
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
                if (connection != null) {
                    try {
                        connection.close();
                    } catch (SQLException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        public void onLoad() {
            ThreadPoolManager.getInstance().scheduleAtFixedRate(new Runnable() {
                @Override
                public void run() {
                    voteProcessor.start();
                }
            }, 5000, POLLING_DELAY_MINUTES);
        }

    }



    public class ApiClient {
        public List<Vote> fetchVotes() throws Exception {
            System.setProperty("https.protocols", "TLSv1.2");

            String url = String.format(API_URL_TEMPLATE, SERVER_ID);
            HttpURLConnection conn = (HttpURLConnection) new URL(url).openConnection();
            conn.setRequestMethod("GET");

            conn.setRequestProperty("X-API-KEY", API_KEY);
            conn.setRequestProperty("User-Agent", "Mozilla/5.0 (compatible; Java API Client)");

            if (conn.getResponseCode() != 200) {
                throw new RuntimeException("API error: HTTP " + conn.getResponseCode());
            }

            BufferedReader reader = new BufferedReader(new InputStreamReader(conn.getInputStream()));
            StringBuilder response = new StringBuilder();
            String line;
            while ((line = reader.readLine()) != null) {
                response.append(line);
            }
            reader.close();

            Gson gson = new Gson();
            Type listType = new TypeToken<List<Vote>>() {
            }.getType();
            return gson.fromJson(response.toString(), listType);
        }
    }
    public static class DatabaseManager {

        private static Connection connection;

        public DatabaseManager(Connection connection) {
            this.connection = connection;
        }

        // Метод для создания таблицы, если она еще не существует
        public void createTableIfNotExists() throws SQLException {
            String query = "CREATE TABLE IF NOT EXISTS votes (" +
                    "id INT NOT NULL, " +
                    "char_name VARCHAR(255) NOT NULL, " +
                    "vote_time DATETIME NOT NULL, " +
                    "reward_given BOOLEAN DEFAULT FALSE, " +
                    "PRIMARY KEY(id)" +
                    ")";
            Statement stmt = null;
            try {
                stmt = connection.createStatement();
                stmt.execute(query);
            } finally {
                if (stmt != null) {
                    stmt.close();
                }
            }
        }

        // Метод для добавления нового голоса в базу данных
        public boolean addVote(String charName, String voteTime, int id) throws SQLException {
            PreparedStatement ps = null;
            ResultSet rs = null;

            try {
                String checkQuery = "SELECT 1 FROM votes WHERE id = ?";
                ps = connection.prepareStatement(checkQuery);
                ps.setInt(1, id);
                rs = ps.executeQuery();
                if (rs.next()) {
                    return false; // Голос с таким ID уже существует
                }
            } finally {
                if (rs != null) {
                    rs.close();
                }
                if (ps != null) {
                    ps.close();
                }
            }

            try {
                String insertQuery = "INSERT INTO votes (id, char_name, vote_time, reward_given) VALUES (?, ?, ?, FALSE)";
                ps = connection.prepareStatement(insertQuery);
                ps.setInt(1, id);
                ps.setString(2, charName);
                ps.setString(3, voteTime);
                ps.executeUpdate();
            } finally {
                if (ps != null) {
                    ps.close();
                }
            }
            return true;
        }

        public void giveRewards(boolean allowMultipleRewards) throws SQLException {
            PreparedStatement ps = null;
            ResultSet rs = null;

            String today = new SimpleDateFormat("yyyy-MM-dd").format(new Date());

            try {
                String selectQuery = allowMultipleRewards
                        ? "SELECT char_name, id FROM votes WHERE reward_given = FALSE AND DATE(vote_time) = ?"
                        : "SELECT char_name, id FROM votes WHERE reward_given = FALSE AND DATE(vote_time) = ?";
                ps = connection.prepareStatement(selectQuery);
                ps.setString(1, today);
                rs = ps.executeQuery();

                while (rs.next()) {
                    String charName = rs.getString("char_name");
                    int voteId = rs.getInt("id");

                    if (!allowMultipleRewards) {
                        String checkRewardQuery = "SELECT COUNT(*) FROM votes WHERE char_name = ? AND DATE(vote_time) = ? AND reward_given = TRUE";
                        PreparedStatement checkPs = null;
                        ResultSet checkRs = null;
                        try {
                            checkPs = connection.prepareStatement(checkRewardQuery);
                            checkPs.setString(1, charName);
                            checkPs.setString(2, today);
                            checkRs = checkPs.executeQuery();
                            if (checkRs.next() && checkRs.getInt(1) == 0) {
                                if(CheckIpAndHwid(charName)) {
                                    grantReward(charName);
                                }else{
                                    int objId = getCharIdByName(charName);
                                    if(objId > 0 && L2ObjectsStorage.getPlayer(objId) != null){
                                        L2Player player = L2ObjectsStorage.getPlayer(objId);
                                        player.sendMessage("Вы уже сегодня голосовали на этот IP или HWID, попробуйте после 00:00");
                                    }
                                }
                                markRewardAsGiven(charName, today);
                            }else{
                                int objId = getCharIdByName(charName);
                                if(objId > 0 && L2ObjectsStorage.getPlayer(objId) != null){
                                    L2Player player = L2ObjectsStorage.getPlayer(objId);
                                    player.sendMessage("Вы уже сегодня голосовали на этот ник, попробуйте после 00:00");
                                }
                                markRewardAsGiven(charName, today);
                            }
                        } finally {
                            if (checkRs != null) {
                                checkRs.close();
                            }
                            if (checkPs != null) {
                                checkPs.close();
                            }
                        }
                    } else {
                        String checkVoteQuery = "SELECT COUNT(*) FROM votes WHERE id = ? AND reward_given = TRUE";
                        PreparedStatement checkVotePs = null;
                        ResultSet checkVoteRs = null;
                        try {
                            checkVotePs = connection.prepareStatement(checkVoteQuery);
                            checkVotePs.setInt(1, voteId);
                            checkVoteRs = checkVotePs.executeQuery();
                            if (checkVoteRs.next() && checkVoteRs.getInt(1) == 0) {
                                if(CheckIpAndHwid(charName)) {
                                    grantReward(charName);
                                }else{
                                    int objId = getCharIdByName(charName);
                                    if(objId > 0 && L2ObjectsStorage.getPlayer(objId) != null){
                                        L2Player player = L2ObjectsStorage.getPlayer(objId);
                                        player.sendMessage("Вы уже сегодня голосовали на этот IP или HWID, попробуйте после 00:00");
                                    }
                                }
                                markRewardAsGiven(charName, today);
                            }
                        } finally {
                            if (checkVoteRs != null) {
                                checkVoteRs.close();
                            }
                            if (checkVotePs != null) {
                                checkVotePs.close();
                            }
                        }
                    }
                }
            } finally {
                if (rs != null) {
                    rs.close();
                }
                if (ps != null) {
                    ps.close();
                }
            }
        }




            public boolean CheckIpAndHwid(String charName) throws SQLException {
                String ip = null;
                String hwid = null;

                String getAccountNameQuery = "SELECT account_name, LastHWID FROM characters WHERE char_name = ?";
                String getAccountDetailsQuery = "SELECT last_ip FROM accounts WHERE login = ?";

                if (connection == null || connection.isClosed()) {
                    throw new IllegalStateException("Database connection is not initialized or closed.");
                }

                PreparedStatement ps1 = null;
                ResultSet rs1 = null;
                PreparedStatement ps2 = null;
                ResultSet rs2 = null;

                try {
                    // Шаг 1: Получение account_name
                    String accountName = null;
                    ps1 = connection.prepareStatement(getAccountNameQuery);
                    ps1.setString(1, charName);
                    rs1 = ps1.executeQuery();
                    if (rs1.next()) {
                        accountName = rs1.getString("account_name");
                        hwid = rs1.getString("LastHWID");
                    }

                    if (accountName == null) {
                        System.err.println("Account name not found for character: " + charName);
                        return false;
                    }

                    // Шаг 2: Получение lastIP
                    ps2 = connection.prepareStatement(getAccountDetailsQuery);
                    ps2.setString(1, accountName);
                    rs2 = ps2.executeQuery();
                    if (rs2.next()) {
                        ip = rs2.getString("last_ip");
                    }

                    // Форматтер для работы с датами в Java 6
                    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd");
                    String todayDate = dateFormat.format(new Date());

                    // Шаг 3: Проверка IP и HWID
                    if (ENABLE_CHECK_IP && ip != null) {
                        String key = "l2vote_ip_" + ip;
                        String storedDate = ServerVariables.getString(key, "null");
                        if (storedDate != null && !storedDate.equals("null")) {
                            if (storedDate.equals(todayDate)) {
                                return false;
                            }
                        }
                        ServerVariables.set(key, todayDate);
                    }

                    if (ENABLE_CHECK_HWID && hwid != null) {
                        String key = "l2vote_hwid_" + hwid;
                        String storedDate = ServerVariables.getString(key, "null");
                        if (storedDate != null && !storedDate.equals("null")) {
                            if (storedDate.equals(todayDate)) {
                                return false;
                            }
                        }
                        ServerVariables.set(key, todayDate);
                    }

                    return true;
                } catch (SQLException e) {
                    e.printStackTrace();
                    throw e;
                } catch (Exception e) {
                    e.printStackTrace();
                    return false;
                } finally {
                    // Закрываем ресурсы вручную (обязательно для Java 6 и ниже)
                    if (rs1 != null) try { rs1.close(); } catch (SQLException ignored) {}
                    if (ps1 != null) try { ps1.close(); } catch (SQLException ignored) {}
                    if (rs2 != null) try { rs2.close(); } catch (SQLException ignored) {}
                    if (ps2 != null) try { ps2.close(); } catch (SQLException ignored) {}
                }
            }








        public static int getCharIdByName(String charName) throws SQLException {
            String query = "SELECT obj_Id FROM characters WHERE char_name = ?";
            PreparedStatement ps = null;
            ResultSet rs = null;

            try {
                ps = connection.prepareStatement(query);
                ps.setString(1, charName);
                rs = ps.executeQuery();
                if (rs.next()) {
                    return rs.getInt("obj_Id");
                }
            } finally {
                if (rs != null) {
                    rs.close();
                }
                if (ps != null) {
                    ps.close();
                }
            }
            return 0;
        }

        public static void addItemToDelayed(int ownerId, int itemId, int count, int enchantLevel, int paymentStatus) throws SQLException {
            String insertQuery = "INSERT INTO character_donate (obj_Id, item_id, count, enchant, given) VALUES (?, ?, ?, ?, ?)";
            PreparedStatement ps = null;

            try {
                ps = connection.prepareStatement(insertQuery);
                ps.setInt(1, ownerId);
                ps.setInt(2, itemId);
                ps.setInt(3, count);
                ps.setInt(4, enchantLevel);
                ps.setInt(5, paymentStatus);
                ps.executeUpdate();
            } finally {
                if (ps != null) {
                    ps.close();
                }
            }
        }

        private static void grantReward(String charName) throws SQLException {
            int obj_id = getCharIdByName(charName);
            if (obj_id > 0) {
                if (!L2VoteRewardItemRandom) {
                    for (String string : L2VoteRewardItemId) {
                        String[] str = string.split(":");
                        int id = Integer.parseInt(str[0]);
                        int count = Integer.parseInt(str[1]);
                        addItemToDelayed(obj_id, id, count, 0, 0);
                        LogManagerVote.getInstance().info("Выдача награды игроку (ID: " + id + ", Name: " + charName + ")");
                    }
                } else {
                    Random random = new Random();
                    int index = random.nextInt(L2VoteRewardItemId.length);
                    String[] strings = L2VoteRewardItemId[index].split(":");
                    int id = Integer.parseInt(strings[0]);
                    long count = Integer.parseInt(strings[1]);
                    addItemToDelayed(obj_id, id, (int) count, 0, 0);
                    LogManagerVote.getInstance().info("Выдача награды игроку (ID: " + id + ", Name: " + charName + ")");
                }
            } else {
                LogManagerVote.getInstance().error("Ошибка при выдаче наград: Игрок с ником - " + charName + " не найден!");
            }
        }

        private void markRewardAsGiven(String charName, String today) throws SQLException {
            String updateQuery = "UPDATE votes SET reward_given = TRUE WHERE char_name = ? AND DATE(vote_time) = ?";
            PreparedStatement ps = null;

            try {
                ps = connection.prepareStatement(updateQuery);
                ps.setString(1, charName);
                ps.setString(2, today);
                ps.executeUpdate();
            } finally {
                if (ps != null) {
                    ps.close();
                }
            }
        }

        public void close() throws SQLException {
            if (connection != null) {
                connection.close();
            }
        }
    }



    public static class LogManagerVote {

        private static LogManagerVote instance; // Экземпляр Singleton
        private Logger logger;

        // Закрытый конструктор, чтобы никто не мог создать объект напрямую
        private LogManagerVote() {
            FileHandler fileHandler = null;
            try {
                // Получаем абсолютный путь к текущей директории
                String logFilePath = new File("").getAbsolutePath() + "/votes.log";

                // Создаем логгер
                logger = Logger.getLogger(LogManagerVote.class.getName());

                // Очищаем существующие обработчики (чтобы избежать вывода в консоль)
                Handler[] handlers = logger.getHandlers();
                for (Handler handler : handlers) {
                    logger.removeHandler(handler);
                }

                // Устанавливаем FileHandler для записи логов в файл
                fileHandler = new FileHandler(logFilePath, true); // append = true для добавления в конец файла
                fileHandler.setFormatter(new SimpleFormatter());  // Устанавливаем простой формат вывода

                // Добавляем обработчик в логгер
                logger.addHandler(fileHandler);
                logger.setLevel(Level.INFO);  // Устанавливаем уровень логирования

            } catch (IOException e) {
                System.err.println("Ошибка при настройке логгера: " + e.getMessage());
                if (fileHandler != null) {
                    fileHandler.close();
                }
            }
        }

        // Метод для получения единственного экземпляра LogManager
        public static synchronized LogManagerVote getInstance() {
            if (instance == null) {
                instance = new LogManagerVote(); // Создаем экземпляр, если он еще не был создан
            }
            return instance;
        }

        // Метод для записи информационного лога
        public void info(String message) {
            logger.info(message);
        }

        // Метод для записи ошибки
        public void error(String message) {
            logger.severe(message);
        }

        // Метод для записи предупреждения
        public void warning(String message) {
            logger.warning(message);
        }

        // Метод для записи общего лога
        public void log(Level level, String message) {
            logger.log(level, message);
        }
    }

    public static class Vote {
        private int id;
        private String charName;
        private String voteTime;

        // Геттеры и сеттеры
        public int getId() {
            return id;
        }

        public void setId(int id) {
            this.id = id;
        }

        public String getCharName() {
            return charName;
        }

        public void setCharName(String charName) {
            this.charName = charName;
        }

        public String getVoteTime() {
            return voteTime;
        }

        public void setVoteTime(String voteTime) {
            this.voteTime = voteTime;
        }

        @Override
        public String toString() {
            return "Vote{" +
                    "id=" + id +
                    ", charName='" + charName + '\'' +
                    ", voteTime='" + voteTime + '\'' +
                    '}';
        }
    }


}
